import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;

public class MountainCanvas extends BCanvas{
  public static boolean debug = true;
  public static final int WIRE_FRAME = 0;
  public static final int ZBUF       = 1;
  public static final int PERS       = 0;
  public static final int PARA       = 1;

  public Color back_color = Color.black;
  public Color wire_color = Color.white;
  public Color mount_color = new Color(140,130,120);

  public int Ia = (new Color(80,85,85)).getRGB();
  public int Il = (new Color(255,255,255)).getRGB();
  public Vec  L = new Vec(0.0,-1.0, 0.0);
  public Vec  S = new Vec(0.0,-1.0,0.0); // Snow Vector


  public MountainStrip   mount;
  public double rot_x = Math.PI/12.0;
  public double rot_y = Math.PI/6.0; 
  public double init_vp  = 2100;
  public double init_prp = 2500;
  public double vp  = init_vp;
  public double prp = init_prp;
  public int    prev_draw_mode = WIRE_FRAME;
  public int    draw_mode      = WIRE_FRAME;
  public int    proj_mode      = PERS;
  public double surface_roughness = 0.25;
  public double drot = Math.PI/20.0;
  public boolean isSnow    = true;
  public double  snow_rate    = 0.6;
  public double snow_strength = 0.2;
  public int mouse_d_x;
  public int mouse_d_y;

  public int orgx, orgy;
  public int width;
  public int height;

  public void initMount(){
    L = new Vec(0.0, -1.0, 0.0);
    S = new Vec(0.0, -1.0, 0.0); 
    mount     = new MountainStrip(-1000,-1000,1000,1000, -50, 500, 3,3);
    mount.setAllColor(mount_color);
    mount.setSnow(isSnow, snow_rate,snow_strength);
    mount.randomizeElevation(surface_roughness);
    rotate(rot_y, rot_x);
    vp  = init_vp;
    prp = init_prp;
  }

  public MountainCanvas(int width, int height){
    super(width, height);
    this.width = width;
    this.height = height;

    initMount();

    if(proj_mode == PERS){
      mount.calcPerspectiveProjection(vp,prp);
    }
    else if( proj_mode == PARA){
      mount.calcParallelProjection(vp);
    }
  }

  public void paint(Graphics g){
    if( offs != null ){
      g.drawImage(offs,0,0,this);
    }
  }
    
  public void makeOffScreen(){
    offs  = this.createImage(size().width, size().height);
    offsg = offs.getGraphics();
    /* $BCf1{$,(B(0,0) $B$K$J$k$h$&$K$7$F$*$/(B */
    orgx = size().width/2;
    orgy = size().height/2;

    offsg.translate(orgx, orgy);
  }
 
  public void draw_clear(){
    offsg.setColor(back_color);
    // jdk1.1 $B$G$O(B size() --> getSize()
    offsg.fillRect(-orgx,-orgy, size().width, size().height);
  }


  public void draw(){
    if(draw_mode == WIRE_FRAME){
      offsg.setColor(wire_color);
      mount.drawWireFrame(offsg);
    }
    else if( draw_mode == ZBUF){
      MemoryImageSource mis =
	mount.drawZBuffer(offsg, -10000000.0, back_color.getRGB(),
			  - width/2, - height/2,
			  width/2,  height/2,
			  Ia, Il, L, S,vp,prp);
      Toolkit tk = Toolkit.getDefaultToolkit();
      Image image = tk.createImage(mis);
      offsg.drawImage(image,- width/2, - height/2,this);
    }
  }
			
 

  public void clear_mountain(){
    initMount();   
    
    if(proj_mode == PERS){
      mount.calcPerspectiveProjection(vp,prp);
    }
    else if( proj_mode == PARA){
      mount.calcParallelProjection(vp);
    }
   
  }

  public void set_roughness(double roughness){
    surface_roughness = roughness;
  }

  public void divide(){
    mount.divide(surface_roughness);
    mount.setSnow(isSnow, snow_rate,snow_strength);
    if(proj_mode == PERS){
      mount.calcPerspectiveProjection(vp,prp);
    }
    else if( proj_mode == PARA){
      mount.calcParallelProjection(vp);
    }
  
  }

  public void redraw(){
    draw_clear();
    draw();
    paint(getGraphics());
  }

  public void setMountColor(Color color){
    mount.setAllColor(color);
  }

  public void setSnow(boolean snow_covered, double snow_rate,double snow_strength){
    this.isSnow = isSnow;
    this.snow_rate = snow_rate;
    this.snow_strength = snow_strength;
    mount.setSnow(snow_covered,snow_rate,snow_strength);
    
  }

  public void rotate(double rot_x, double rot_y){
      mount.yAxisRotate(rot_y);
      mount.xAxisRotate(rot_x);
      L.xAxisRotate(rot_x);
      L.yAxisRotate(rot_y);
      S.xAxisRotate(rot_x);
      S.yAxisRotate(rot_y);
  }
      
  public void zoom(double z){
    vp  -= z;
    prp -= z;
    if(proj_mode == PERS){
      mount.calcPerspectiveProjection(vp,prp);
    }
    else if( proj_mode == PARA){
      mount.calcParallelProjection(vp);
    }
    
    draw_clear();
    draw();
    paint(getGraphics());
  }

  public void setDrawMode(int mode){
    draw_mode = mode;
    draw_clear();
    draw();
    paint(getGraphics());
  }

  public void setProjMode(int mode){
    proj_mode = mode;
     if(proj_mode == PERS){
      mount.calcPerspectiveProjection(vp,prp);
    }
    else if( proj_mode == PARA){
      mount.calcParallelProjection(vp);
    }
    draw_clear();
    draw();
    paint(getGraphics());
  }

  public boolean mouseDown(Event e, int x, int y){
    prev_draw_mode = draw_mode;
    mouse_d_x = x;
    mouse_d_y = y;
    return false;
  }

  public boolean mouseDrag(Event e, int x, int y){
    draw_mode = WIRE_FRAME;

    double m = Math.abs( (y - mouse_d_y)/(double)(x - mouse_d_x) );
    if( m > 1 ){
      if( y - mouse_d_y < 0 ){
	mount.xAxisRotate( - drot );
	L.xAxisRotate( - drot );
	S.xAxisRotate( - drot );
      }
      else{
	mount.xAxisRotate(  drot );
	L.xAxisRotate( drot );
	S.xAxisRotate( drot );
      }
    }
    else{
      if( x - mouse_d_x < 0 ){
	mount.yAxisRotate( - drot );
	L.xAxisRotate( - drot );
	S.xAxisRotate( - drot );
      }
      else{
	mount.yAxisRotate( drot );
	L.xAxisRotate( drot );
	S.xAxisRotate( drot );
      }
    }
   
  
    if(proj_mode == PERS){
      mount.calcPerspectiveProjection(vp,prp);
    }
    else if( proj_mode == PARA){
      mount.calcParallelProjection(vp);
    }
   
    draw_clear();
    draw();
    paint(getGraphics());
    mouse_d_x = x;
    mouse_d_y = y;
    
    return false;
  }

  public boolean mouseUp(Event e, int x, int y){
    draw_mode = prev_draw_mode;
    draw_clear();
    draw();
    paint(getGraphics());
    return false;
  }
  
}

